<?php
namespace Pinpoint\Conesso\Model\Api\Management\Process;

use Exception;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Api\SearchCriteria;
use Pinpoint\Conesso\Api\Data\ProductSyncInterface;
use Pinpoint\Conesso\Api\Repository\ProductsSyncRepositoryInterface;
use Pinpoint\Conesso\Api\Transfer\RequestInterface;
use Pinpoint\Conesso\Model\Api\Management\Log\Products\LogProductInterface;
use Pinpoint\Conesso\Model\Api\Management\Transfer\Request\ProductRequest;
use Pinpoint\Conesso\Model\Config\ProductSettings;
use Pinpoint\Conesso\Model\ResourceModel\SyncProducts\CollectionFactory;

class Products extends AbstractProcess
{
    /**
     * @var ProductsSyncRepositoryInterface
     */
    private ProductsSyncRepositoryInterface $productsSyncRepository;

    /**
     * @var ProductRepositoryInterface
     */
    private ProductRepositoryInterface $productRepository;

    /**
     * @var LogProductInterface
     */
    private LogProductInterface $logProduct;
    private CollectionFactory $syncProductsCollectionFactory;
    private ProductSettings $productSettings;

    /**
     * @param ProductsSyncRepositoryInterface $productsSyncRepository
     * @param ProductRepositoryInterface $productRepository
     * @param LogProductInterface $logProduct
     * @param CollectionFactory $syncProductsCollectionFactory
     * @param ProductSettings $productSettings
     * @param CreateContext $context
     */
    public function __construct(
        ProductsSyncRepositoryInterface $productsSyncRepository,
        ProductRepositoryInterface $productRepository,
        LogProductInterface $logProduct,
        CollectionFactory $syncProductsCollectionFactory,
        ProductSettings $productSettings,
        CreateContext $context
    ) {
        $this->productsSyncRepository = $productsSyncRepository;
        $this->productRepository = $productRepository;
        $this->logProduct = $logProduct;
        $this->syncProductsCollectionFactory = $syncProductsCollectionFactory;
        $this->productSettings = $productSettings;
        parent::__construct($context);
    }

    /**
     * @param array $params
     * @return bool
     */
    public function synchronizeData(array $params = []) : bool
    {
        $batchSize = $this->productSettings->getBatchSize() ?? 75;
        $items = $this->productsSyncRepository->getList($this->getSearchCriteria($batchSize));
        $this->getRequestProvider()->sendRequest(['items' => $items]);
        return true;
    }

    /**
     * @param int $batchSize
     * @return SearchCriteria
     */
    protected function getSearchCriteria(int $batchSize = 75): SearchCriteria
    {
        return $this->searchCriteriaBuilder
                ->addFilter(ProductSyncInterface::IS_SYNCED, '0')
                ->create()->setPageSize($batchSize);
    }

    /**
     * Actions:
     * 1. Truncate `connesso_product_sync`
     * 2. Add all products to connesso_product_sync table
     * 3. Sync all the products with Conesso
     * 4. Truncate the table again
     * @return void
     */
    public function migrateData(): void
    {
        try {
            $this->truncateSyncTable();
            $products = $this->getAllProductsInMagento();

            foreach($products as $product) {
                $this->logProduct->logProduct($product,ProductSyncInterface::EVENT_TYPE_SAVE);
            }
            $this->synchronizeData();
            $this->truncateSyncTable();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return RequestInterface
     */
    private function getRequestProvider(): RequestInterface
    {
        return $this->requestProvider->getRequestProvider(ProductRequest::class);
    }

    /**
     * Filter by product creation date
     * @return array
     */
    private function getAllProductsInMagento(): array
    {
        $criteria = $this->searchCriteriaBuilder
            ->addFilter(ProductInterface::CREATED_AT, $this->productSettings->getSyncFromDate(), 'gteq')
            ->create();
        return $this->productRepository->getList($criteria)->getItems();
    }

    /**
     * @return void
     */
    private function truncateSyncTable(): void
    {
        $collection = $this->syncProductsCollectionFactory->create();
        $connection = $collection->getConnection();
        $tableName = $collection->getMainTable();
        $connection->truncateTable($tableName);
    }
}
