<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Pinpoint\Conesso\Observer\Catalog;

use Magento\Catalog\Model\Product;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Pinpoint\Conesso\Model\Api\Service as ConessoService;

class ProductSaveAfter implements ObserverInterface
{
    public const MAGENTO_CONESSO_PRODUCT_MAP = [
        'apiReferenceId' => 'sku',
        'name' => 'name',
        'description' => 'description',
        'barcode' => 'sku',
        'sku' => 'sku',
        'uri' => 'url_key',
        'dateCreated' => 'created_at',
        'dateUpdated' => 'updated_at',
        'priceTax' => 'tax_class_id',
        'priceIncTax' => 'price',
        'originalPrice' => 'original_price',
        'price' => 'price',
        'weight' => 'weight',
        'weightUnit' => 'weight_unit',
        'stockQuantity' => 'qty',
        'visibility' => 'visibility',
        'status' => 'status',
        'productLinks' => 'product_links',
        'productOptions' => 'product_options',
        'productImages' => 'media_gallery',
        'productCategories' => 'category_ids',
        'parentVariation' => 'parent_variation',
        'childVariation' => 'child_variation',
        'customFields' => 'custom_fields'
    ];

    public const MAGENTO_CONESSO_VISIBILITY_MAP = [
        1 => 'not-individually',
        2 => 'private',
        3 => 'public',
        4 => 'public'
    ];

    public function __construct(
        private readonly ConessoService $conessoService
    )
    {
        //__construct
    }

    /**
     * Execute observer
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(
        Observer $observer
    ): void
    {
        /** @var Product $product */
        $product = $observer->getProduct();
        $hasChanges = $product->hasDataChanges();

        if (!$hasChanges) {
            return;
        }

        $payload = $this->mapToConessoProduct($product);
        $this->conessoService->products(
            ['json' => $payload],
            $product->isObjectNew() ? 'POST' : 'PUT',
            $product->isObjectNew() ? $product->getSku() : null
        );
    }

    private function mapToConessoProduct(Product $product): array
    {
        $changes = $this->getChangedValues($product);

        $payload = [];
        foreach (self::MAGENTO_CONESSO_PRODUCT_MAP as $conessoKey => $magentoKey) {
            if (isset($changes[$magentoKey])) {
                $payload[$conessoKey] = $changes[$magentoKey];

                if (str_contains($conessoKey, 'price') || $conessoKey === 'weight') {
                    $payload[$conessoKey] = (float)$payload[$conessoKey];
                }
            }
        }

        if (isset($payload['dateCreated'])) {
            $payload['dateCreated'] = $this->formatDateTime($payload['dateCreated']);
        }

        if (isset($payload['dateUpdated'])) {
            $payload['dateUpdated'] = $this->formatDateTime($payload['dateUpdated']);
        }

        if (isset($payload['uri'])) {
            $payload['uri'] = $product->getProductUrl();
        }

        if (isset($payload['visibility'])) {
            $payload['visibility'] = self::MAGENTO_CONESSO_VISIBILITY_MAP[$payload['visibility']];
        }

        if (isset($payload['status'])) {
            $payload['status'] = $product->getStatus() ? 'active' : 'disabled';
        }

        $payload['groupType'] = $product->getTypeId() === 'configurable' ? 'parent-product' : 'standalone-product';

        if ($product->getTypeId() === 'simple') {
            $parents = $product->getTypeInstance()->getParentIdsByChild($product->getId());
            if (count($parents)) {
                $payload['groupType'] = 'child-product';
            }
        }

        return $payload;
    }

    private function getChangedValues(Product $product): array
    {
        $differences = ['sku' => $product->getSku()];

        $editableAttributes = $product->getTypeInstance(true)->getEditableAttributes($product);

        foreach ($editableAttributes as $code => $attribute) {
            if ($product->dataHasChangedFor($code)) {
                $differences[$code] = $product->getData($code);
            }
        }

        return $differences;
    }

    private function formatDateTime(string $dateTime): string
    {
        return date('Y-m-d\TH:i:s\Z', strtotime($dateTime));
    }
}
